/******************************************************************************

JMixNet - A Java MixNet Implementation

Fabiano Castro Pereira
LabSEC - Computer Security Laboratory (www.labsec.ufsc.br)
Informatics and Statistics Department (www.inf.ufsc.br)
Federal University of Santa Catarina  (www.ufsc.br)  
Florianopolis, Santa Catarina, Brazil

JMixNet is a registered trademark of the Federal University of Santa Catarina.
No commercial use of this trademark may be made without prior written
permission of the Federal University of Santa Catarina.\

******************************************************************************/
package br.edu.ufsc.labsec.crypto;

import java.security.cert.*;
import java.util.Collection;
import java.io.*;

/**
 * @author castro
 *
 * Class to generate X509Certificates in many ways.
 */
public class X509CertificateGenerator {
    
    protected static CertificateFactory factory = null;
    
    /**
     * Get the internal X509 factory, instantiating it if it's not available yet. 
     * 
     * @return The internal X509 factory. 
     */
    protected static CertificateFactory getFactory() {
        if (factory == null) {
            try {
                factory = CertificateFactory.getInstance("X509");
            }
            catch (CertificateException e) {
                System.err.println("Error creating CertificateFactory.");
                e.printStackTrace();
            }
        }
        return factory;
    }
    
    /**
     * Generate a collection containing the certificates extracted from the specified file. 
     * 
     * @param fileName The file containing the certificates. 
     * @return A certificate collection.
     * @throws CryptoException If some parse error ocurred.
     */
    public static Collection genCertificates(String fileName) throws CryptoException {
        try {
            return getFactory().generateCertificates(new FileInputStream(fileName));
        }
        catch (CertificateException ex) {
            throw new CryptoException("Certificate parse error during certificate generation.", ex);
        }
        catch (FileNotFoundException ex) {
            throw new CryptoException("Certificates file not found.", ex);
        }
    }
    
    /**
     * Generate a collection containing the certificates extracted from the specified stream. 
     * 
     * @param certStream The stream containing the certificates. 
     * @return A certificate collection.
     * @throws CryptoException If some parse error ocurred.
     */
    public static Collection genCertificates(InputStream certStream) throws CryptoException {
        try {
            return getFactory().generateCertificates(certStream);
        }
        catch (CertificateException ex) {
            throw new CryptoException("Certificate parse error during certificate generation.", ex);
        }
    }
}
