/******************************************************************************

JMixNet - A Java MixNet Implementation

Fabiano Castro Pereira
LabSEC - Computer Security Laboratory (www.labsec.ufsc.br)
Informatics and Statistics Department (www.inf.ufsc.br)
Federal University of Santa Catarina  (www.ufsc.br)  
Florianopolis, Santa Catarina, Brazil

JMixNet is a registered trademark of the Federal University of Santa Catarina.
No commercial use of this trademark may be made without prior written
permission of the Federal University of Santa Catarina.\

******************************************************************************/
package br.edu.ufsc.labsec.jmixnet;

import java.io.*;
import java.nio.ByteBuffer;
import java.net.*;

/**
 * @author castro
 *
 * Default JMixNet message deliverer. 
 */
public class JMixNetDeliverer implements MessageDeliverer {

    protected PrintStream errorReport = System.err;
    protected InetAddress receiver;
    protected byte[] addressBuf = new byte[4];
    protected short receiverPort;
    protected short meaningfulMsgSize;
    protected Socket receiverSocket;
    
    /* (non-Javadoc)
     * @see br.edu.ufsc.labsec.jmixnet.MessageDeliverer#deliverMsg(java.nio.ByteBuffer)
     */
    public void deliverMsg(ByteBuffer msgBuffer) {
        try {
            //get the meaningful message size
            meaningfulMsgSize = msgBuffer.getShort();
            
            //get the raw address and port
            msgBuffer.get(addressBuf, 0, 4);
            receiver = InetAddress.getByAddress(addressBuf);
            receiverPort = msgBuffer.getShort();
            
            //connect and deliver
            receiverSocket = new Socket(receiver, receiverPort);
            receiverSocket.getOutputStream().write(msgBuffer.array(), 8, meaningfulMsgSize - 6);
            receiverSocket.close();
        }
        catch (UnknownHostException ex) {
            errorReport.println("DELIVERY ERROR - Unknown host: " + receiver.toString() + ". Details: " + ex.getMessage());
        } 
        catch (IOException ex) {
            errorReport.println("DELIVERY ERROR - Unable to communicate with host: " + receiver.toString() + ". Details: " + ex.getMessage());
        }
    }
}