/******************************************************************************

JMixNet - A Java MixNet Implementation

Fabiano Castro Pereira
LabSEC - Computer Security Laboratory (www.labsec.ufsc.br)
Informatics and Statistics Department (www.inf.ufsc.br)
Federal University of Santa Catarina  (www.ufsc.br)  
Florianopolis, Santa Catarina, Brazil

JMixNet is a registered trademark of the Federal University of Santa Catarina.
No commercial use of this trademark may be made without prior written
permission of the Federal University of Santa Catarina.\

******************************************************************************/
package br.edu.ufsc.labsec.jmixnet;

import java.awt.*;
import java.awt.event.*;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.Date;

import javax.swing.*;

/**
 * @author castro
 *
 */
public class ServerGUI implements ActionListener {
    
    protected static JFrame mainWindow;
    protected static JLabel lbConnectedClients;
    protected static JLabel lbRefusedCons;
    protected static JLabel lbStoredMsg;
    protected static JLabel lbBadMsgs;
    protected static JLabel lbPerformance;
    protected static JButton btService;
    protected static JButton btExit;
    protected static JMixNetServer server;
    
    protected static Timer timer;
    protected static final int TIMER_PERIOD = 1000;
    protected static int lastMsgCount = 0;
    protected static long lastPerfMeasure = 0;
    protected static double elapsedTime;
    protected static Double performance;

    public void createComponents(Container contentPane) {
        
        Font fonte = new Font("Tahoma", Font.PLAIN, 11);
        JLabel label;

        contentPane.setLayout(new GridLayout(0, 2, 5, 5));
        
        label = new JLabel(" Connected Clients:");
        label.setFont(fonte);
        contentPane.add(label);
        
        lbConnectedClients = new JLabel("0");
        lbConnectedClients.setFont(fonte);
        contentPane.add(lbConnectedClients);

        label = new JLabel(" Refused Connections:");
        label.setFont(fonte);
        contentPane.add(label);
        
        lbRefusedCons = new JLabel("0");
        lbRefusedCons.setFont(fonte);
        contentPane.add(lbRefusedCons);

        label = new JLabel(" Stored Messages:");
        label.setFont(fonte);
        contentPane.add(label);
        
        lbStoredMsg = new JLabel("0");
        lbStoredMsg.setFont(fonte);
        contentPane.add(lbStoredMsg);

        label = new JLabel(" Bad Messages:");
        label.setFont(fonte);
        contentPane.add(label);
        
        lbBadMsgs = new JLabel("0");
        lbBadMsgs.setFont(fonte);
        contentPane.add(lbBadMsgs);

        label = new JLabel(" Performance:");
        label.setFont(fonte);
        contentPane.add(label);
        
        lbPerformance = new JLabel("0 mgs/s");
        lbPerformance.setFont(fonte);
        contentPane.add(lbPerformance);

        btService = new JButton("Start");
        btService.addActionListener(this);
        contentPane.add(btService);

        btExit = new JButton("Exit");
        btExit.addActionListener(this);
        contentPane.add(btExit);
        
        timer = new Timer(TIMER_PERIOD, this);
    }

    public void actionPerformed(ActionEvent e) {
        Object src = e.getSource();
        
        // Service "Start" and "Stop" button
        if (src == btService) {
            if (btService.getText().equals("Start")) { 
                btService.setText("Stop");
                server.start();
                timer.start();
            }
            else {
                //accounting
                lastPerfMeasure = new Date().getTime();
                timer.stop();
                
                //finish
                btService.setText("Start");
                server.finishService();
            }
        }
        // "Exit" button
        else if (src == btExit) {
            System.exit(0);
        }
        // Timer
        else if (src == timer) {
            //measure performance
            elapsedTime = (new Date().getTime() - lastPerfMeasure) / 1000.0;
            lastPerfMeasure = new Date().getTime();
            performance = new Double((server.getProcessedMsgCount() - lastMsgCount) / elapsedTime);
            lastMsgCount = server.getProcessedMsgCount();
            lbPerformance.setText(performance.intValue() + " msg/s");
            
            //other stats
            //UNDO lbBadMsgs.setText(server.getBadMsgCount() + "");
            lbStoredMsg.setText(server.getStoredMsgCount() + "");
            lbConnectedClients.setText(server.getConnectedClientes() + "");
            lbRefusedCons.setText(server.getRefusedConnections() + "");
        }
    }

    private static void createAndShowGUI() {
        //create and set up the window
        mainWindow = new JFrame("JMixNet Server");
        mainWindow.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        mainWindow.setLocation(500,300);
        mainWindow.setResizable(false);

        ServerGUI serverGUI = new ServerGUI();
        serverGUI.createComponents(mainWindow.getContentPane());

        //Display the window.
        mainWindow.pack();
        mainWindow.setVisible(true);
    }

    public static void main(String[] args) throws FileNotFoundException, JMixNetException {
        JMixNetServer server;

        //create JMixNet server according to command line args
        if (0 == args.length) 
            server = new JMixNetServer();
        else
            server = new JMixNetServer(new FileInputStream(args[0]));

        //creating and showing GUI.
        javax.swing.SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                createAndShowGUI();
            }
        });
        ServerGUI.setServer(server);
    }

    public static void setServer(JMixNetServer server) {
        ServerGUI.server = server;
    }
}