/******************************************************************************

JMixNet - A Java MixNet Implementation

Fabiano Castro Pereira
LabSEC - Computer Security Laboratory (www.labsec.ufsc.br)
Informatics and Statistics Department (www.inf.ufsc.br)
Federal University of Santa Catarina  (www.ufsc.br)  
Florianopolis, Santa Catarina, Brazil

JMixNet is a registered trademark of the Federal University of Santa Catarina.
No commercial use of this trademark may be made without prior written
permission of the Federal University of Santa Catarina.\

******************************************************************************/
package br.edu.ufsc.labsec.util;

import java.io.*;
import java.util.*;

/**
 * @author castro
 *
 * Controls the reading e writing of configuration from/into properties files
 */
public class ConfigManager extends Properties {
   
    protected String configFileName;
    protected String readValue;

    /**
     * Creates a config manager loading the specified file.
     */
    public ConfigManager(String fileName) throws ConfigException {
        
        FileInputStream configFile;
        
        try {
            configFileName = fileName;
            configFile = new FileInputStream(fileName);
            load(configFile);
            configFile.close();        
        }
        catch (FileNotFoundException ex) {
            throw new ConfigFileNotFoundException(configFileName);
        }
        catch (IOException ex) {
            throw new ConfigException("Error reading the configuration file."); 
        }
        
    }
       
    /**
     * Creates a config manager loading the specified config stream.
     */
    public ConfigManager(InputStream configStream) throws ConfigException {
        try {
            load(configStream);
        }
        catch (IOException ex) {
            throw new ConfigException("Error reading the configuration."); 
        }
    }
       
    /**
     * Stores the configuration back to original file.
     */
    public void store() throws IOException {
        FileOutputStream out = new FileOutputStream(configFileName);
        store(out, "");
        out.close();
    }
    
    /**
     * Stores the configuration with a header back to original file.
     */
    public void store(String header) throws IOException {
        FileOutputStream out = new FileOutputStream(configFileName);
        store(out, header);
        out.close();
    }
 
    /**
     * Get a property as an int.
     */
    public int getInt(String key) {
        return Integer.parseInt(getProperty(key));
    }
    
    /**
     * Get a property as an int, returns the default value argument if the property is not found
     */
    public int getInt(String key, int defaultValue) {
        readValue = getProperty(key);
        
        if (readValue == null)
            return defaultValue;
        else
            return Integer.parseInt(readValue);
    }
    
    /**
     * Get a property as a string.
     */
    public String getString(String key) {
        return getProperty(key);
    }
    
    /**
     * Get a property as a string, returns the default value argument if the property is not found
     */
    public String getString(String key, String defaultValue) {
        readValue = getProperty(key);
        
        if (readValue == null)
            return defaultValue;
        else
            return readValue;
    }   
}